@@ -33,7 +33,6 @@ def bin_badrate(df, col_name, target):
     return redf
 
 
-
 def chi2(regroup):
     """
     计算并返回两个相邻组的卡方值
@@ -85,7 +84,6 @@ def chi2(regroup):
     return chi2_bad + chi2_good
 
 
-
 def chi2sum_id(regroup, field='all'):
     """
     找出需要待合并的分箱位置(id)
@@ -245,7 +243,6 @@ def refresh_vals_dict(dict_vals_ori, dict_vals_proc):
     return dict_res
 
 
-
 def merge_neighbour(regroup, id_target, feat_vals_dict):
     """
     合并分箱
@@ -262,7 +259,6 @@ def merge_neighbour(regroup, id_target, feat_vals_dict):
     返回刷新组序列之后的取值dict
     """
 
-
     # 先初始化索引值,方便取值,也方便后续更新refresh_val_2bin以确保一一对应, 这里可以理解为GroupNO
     rg = regroup.reset_index()
     # 取得对应字典, 注意, 这里的key和value需要翻转, 用于refresh便捷
@@ -646,7 +642,7 @@ def merge_intervals(dict_vals_to_bins):
     # 合并特殊值
     res_dict_vals_to_bins.update(sp_dict_vals_to_bins)
     # 做个排序
-    res_dict_vals_to_bins = {k: v for k,v in sorted(res_dict_vals_to_bins.items(), key=lambda x: x[1])}
+    res_dict_vals_to_bins = {k: v for k, v in sorted(res_dict_vals_to_bins.items(), key=lambda x: x[1])}
     return res_dict_vals_to_bins
 
 
@@ -673,7 +669,6 @@ def value_to_intervals(value, dict_valstoinv):
 
     """
 
-
     for key in list(dict_valstoinv.keys()):
         if isinstance(key, str):
             label_l = key.split(',')[0]
@@ -703,7 +698,6 @@ def value_to_intervals(value, dict_valstoinv):
     raise Exception(str_exception)
 
 
-
 def chi2_cutting_discrete(df_data, feat_list, target,
                           special_feat_val={},
                           max_intervals=8, min_pnt=0.05,
@@ -939,7 +933,6 @@ def chi2_cutting_discrete(df_data, feat_list, target,
             regroup_woe_iv = regroup_special_merge(regroup_woe_iv, regroup_special=regroup_special)
             # 注意, 如果最后的regroup存在分组占比数小于min_pnt的, 需要提示, 但不做处理
 
-
         # 计算分组的woe和iv
         # 注意这里用的参数是regroup_woe_iv
         # 这个时候只有num_feat_vals、num_feat_bad、bad_rate三个字段
@@ -955,10 +948,13 @@ def chi2_cutting_discrete(df_data, feat_list, target,
     return dict_discrete_feat_to_bins, dict_discrete_iv, dict_discrete_woe
 
 
-def chi2_cutting_continuous(df_data, feat_list, target,
+def chi2_cutting_continuous(df_data,
+                            feat_list,
+                            target,
                             discrete_more_feats=[],
                             special_feat_val={},
-                            max_intervals=5, min_pnt=0.05,
+                            max_intervals=5,
+                            min_pnt=0.05,
                             mono_expect={}):
     """
     连续性特征分箱
@@ -1047,8 +1043,10 @@ def chi2_cutting_continuous(df_data, feat_list, target,
             intervals -= len(special_feat_val[feat])
             # 复制一份特殊值的df
             df_special = df.loc[df[feat].isin(special_feat_val[feat])]
+
             # 同时也创建feat + '_bins'列
             df_special[feat + '_bins'] = df_special[feat + '_transf']
+
             # 更新df(剔除特殊值)
             df = df.loc[~df[feat].isin(special_feat_val[feat])]
 
@@ -1093,8 +1091,8 @@ def chi2_cutting_continuous(df_data, feat_list, target,
 
             # 基于卡方进行合并, 更新取值字典
             dict_vals_to_bins = merge_neighbour(regroup_badrate,
-                                              id_target=id_merge_badrate,
-                                              feat_vals_dict=dict_vals_to_bins)
+                                                id_target=id_merge_badrate,
+                                                feat_vals_dict=dict_vals_to_bins)
             # 重新更新df和regroup_badrate
             df[feat + '_badrate'] = df[feat + '_bins'].map(dict_vals_to_bins)
             regroup_badrate = bin_badrate(df, col_name=feat + '_badrate', target=target)
@@ -1116,8 +1114,8 @@ def chi2_cutting_continuous(df_data, feat_list, target,
                                                           check_object='min_pnt')
             # 开始进行合并, 更新分组取值
             dict_vals_to_bins = merge_neighbour(regroup_min_pnt,
-                                              id_target=id_merge_pnt,
-                                              feat_vals_dict=dict_vals_to_bins)
+                                                id_target=id_merge_pnt,
+                                                feat_vals_dict=dict_vals_to_bins)
             # 对df进行更新
             df[feat + '_min_pnt'] = df[feat + '_bins'].map(dict_vals_to_bins)
             # 更新regroup_min_pnt
@@ -1141,8 +1139,8 @@ def chi2_cutting_continuous(df_data, feat_list, target,
                 regroup_mono, id_merge_mono = order_regroup(regroup_mono, feat_order=True)
                 # 更新feat_valToBins
                 dict_vals_to_bins = merge_neighbour(regroup_mono,
-                                                  id_target=id_merge_mono,
-                                                  feat_vals_dict=dict_vals_to_bins)
+                                                    id_target=id_merge_mono,
+                                                    feat_vals_dict=dict_vals_to_bins)
                 # 更新df, feat + '_mono'
                 df[feat + '_mono'] = df[feat + '_bins'].map(dict_vals_to_bins)
                 # 更新regroup_mono
@@ -1188,7 +1186,7 @@ def chi2_cutting_continuous(df_data, feat_list, target,
             # 重新刷新&排序dict_valsToBins
             dict_vals_to_bins = regroup_tf[feat + '_groupno'].to_dict()
             dict_vals_to_bins = {k: v for k, v in sorted(dict_vals_to_bins.items(),
-                                                       key=lambda x: x[1])}
+                                                         key=lambda x: x[1])}
             dict_contin_feat_to_bins[feat] = dict_vals_to_bins
         else:
             dict_vals_to_bins = merge_intervals(dict_vals_to_bins)
@@ -1200,6 +1198,7 @@ def chi2_cutting_continuous(df_data, feat_list, target,
 
     return dict_contin_feat_to_bins, dict_contin_iv, dict_contin_woe
 
+
 def model_roc_auc(y_true, y_proba, is_plot=False, dict_plot_params=None):
     """
     绘制roc曲线
@@ -1273,7 +1272,7 @@ def model_roc_auc(y_true, y_proba, is_plot=False, dict_plot_params=None):
     return auc
 
 
-def model_ks(y_true, y_pred, y_proba, is_plot=True, dict_plot_params=None):
+def model_ks(y_true, y_pred, y_proba, is_plot=False, dict_plot_params=None):
     """
     计算模型的ks(Kolmogorov-Smirnov)
 
@@ -1821,7 +1820,7 @@ def plot_score_distribution(y, score, dict_plot_params=None):
     ax[1].legend(fontsize='large')
 
     # 第三个绘制badrate的频数分布
-    score_min, score_max = df['score'].min() // 100 * 100, df['score'].max() // 100 * 100
+    # score_min, score_max = df['score'].min() // 100 * 100, df['score'].max() // 100 * 100
 
     x_linsrange = np.linspace(df['score'].min(), df['score'].max(), 20)
 
@@ -1859,4 +1858,4 @@ def plot_score_distribution(y, score, dict_plot_params=None):
     twinx_ax2.legend(loc='upper right', fontsize='x-large')
 
     # 铺满整个画布
-    fig.tight_layout()
\ No newline at end of file
+    fig.tight_layout()
